<?php

namespace Drupal\ckeditor_backup\Plugin\CKEditorPlugin;

use Drupal\ckeditor\CKEditorPluginBase;
use Drupal\ckeditor\CKEditorPluginConfigurableInterface;
use Drupal\ckeditor\CKEditorPluginContextualInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\editor\Entity\Editor;

/**
 * Defines the "CKEditor Backup" plugin.
 *
 * @CKEditorPlugin(
 *   id = "jsplus_backup",
 *   label = @Translation("CKEditor Backup"),
 *   module = "ckeditor_backup"
 * )
 */
class ckeditor_backup extends CKEditorPluginBase implements CKEditorPluginConfigurableInterface, CKEditorPluginContextualInterface {

  public $plugins = array(
     'jsplus_backup_save' => array(
    'urlDoc' => 'http://js.plus/products/backup/configuration',
    'buttons' => array(array('label' => 'Save backup snapshot')),
    'params' => array(
        array(
            'name' => 'jsplus_dialog_width',
            'default' => 750,
            'type' => 'int',
            'order' => 1000,
            'title' => 'Width of the Restore dialog',
            'hint' => '',
            'widget' => 'text'
        ),
        array(
            'name' => 'jsplus_dialog_height',
            'default' => 550,
            'type' => 'int',
            'order' => 1010,
            'title' => 'Height of the Restore dialog',
            'hint' => '',
            'widget' => 'text'
        ),
        array(
            'name' => 'jsplus_backup_save_empty',
            'default' => FALSE,
            'type' => 'bool',
            'order' => 1020,
            'title' => 'Not save empty snapshots',
            'hint' => 'For save empty snapshots turn on TRUE',
            'widget' => 'checkbox'
        ),
        array(
            'name' => 'jsplus_backup_interval',
            'default' => 30000,
            'type' => 'int',
            'order' => 1030,
            'title' => 'Backup interval in milliseconds',
            'hint' => 'The interval in milliseconds to make new snapshot',
            'widget' => 'text'
        ),
        array(
            'name' => 'jsplus_backup_snapshots_limit',
            'default' => 20,
            'type' => 'int',
            'order' => 1040,
            'title' => 'The maximum number of snapshots',
            'hint' => 'The maximum number of snapshots made the Backup plugin. In case of limit plugin removes the last backup',
            'widget' => 'text'
        ),
        array(
            'name' => 'jsplus_backup_save_before_load',
            'default' => TRUE,
            'type' => 'bool',
            'order' => 1050,
            'title' => 'Save before load',
            'hint' => 'Make a new snapshot before restoring selected',
            'widget' => 'checkbox'
        ),
        array(
            'name' => 'jsplus_backup_add_background_in_footer',
            'default' => TRUE,
            'type' => 'bool',
            'order' => 1070,
            'title' => 'Background of button in footer',
            'hint' => 'In case TRUE the buttons will have a typical background which has any of toolbar button groups',
            'widget' => 'checkbox'
        ),
        array(
            'name' => 'jsplus_backup_add_text_to_load_button',
            'default' => TRUE,
            'type' => 'bool',
            'order' => 1080,
            'title' => 'Add text to load button',
            'hint' => 'Adds "Restore" title to according toolbar button if TRUE',
            'widget' => 'checkbox'
        ),
        array(
            'name' => 'jsplus_backup_date_format',
            'default' => '',
            'type' => 'str',
            'order' => 1090,
            'title' => 'Date format',
            'hint' => 'Defines a format to show a dates in the restore dialog',
            'widget' => 'text'
        ),
        array(
            'name' => 'jsplus_backup_api_key',
            'default' => '',
            'type' => 'str',
            'order' => 2000,
            'title' => 'API key',
            'hint' => 'Specify API key to use API of CKEditor Backup add-on',
            'widget' => 'text'
        )
    )
),
'jsplus_backup_load' => array(
    'urlDoc' => 'http://js.plus/products/backup/configuration',
    'buttons' => array(array('label' => 'Load backup snapshot')),
)
  );

    /**
     * {@inheritdoc}
     */
    public function getButtons() {
        if (!$this->isInstalled())
            return array();

        $buttons = array();
        foreach ($this->plugins as $pluginName => $pluginDef) {

            if (isset($pluginDef['buttons'])) {
                foreach($pluginDef['buttons'] as $buttonName => $buttonDef) {
                    $image = 'https://cdn.n1ed.com/cdn/buttons/' . $pluginName . '.png';
                    $button = array(
                        'label' => $buttonDef['label'],
                        'image' => $image
                    );
                    $buttons[$pluginName] = $button;
                }
            }

        }
        return $buttons;
    }

  /**
   * {@inheritdoc}
   */
  public function getFile() {
    return '/libraries/jsplus_backup/plugin.js';
  }

  /**
   * {@inheritdoc}
   */
  public function getDependencies(Editor $editor) {
    return array();
  }

  /**
   * {@inheritdoc}
   */
  public function getLibraries(Editor $editor) {
    return array();
  }

  /**
   * {@inheritdoc}
   */
  public function isInternal() {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function isEnabled(Editor $editor) {
    return TRUE;
  }

  function getConfigParam($settings, $param, $default, $type, $inside) {
    $name = $param;
    if (isset($settings[$name]) && strlen($settings[$name]) > 0)
        $value = $settings[$name];
    else
        $value = $default;
    if (isset($type) && $type == 'int') {
        $value = intval($value);
    } else if (isset($type) && $type == 'bool') {
        if ($value == '1')
            $value = true;
        else if ($value == '0')
            $value = false;
    } else if (isset($type) && $type == 'json') {
        $value = json_decode($value);
    }
    return $value;
  }

  /**
   * {@inheritdoc}
   */
  public function getConfig(Editor $editor) {
      $settings = array();
      if (isset($editor->getSettings()['plugins']['jsplus_backup']))
        $settings = $editor->getSettings()['plugins']['jsplus_backup'];
      $result = array();
      foreach ($this->plugins as $pluginName => $pluginDef) {
          if (isset($pluginDef['params'])) {
              foreach ($pluginDef['params'] as $paramDef) {
                  $value = $this->getConfigParam($settings, $paramDef['name'], $paramDef['default'], $paramDef['type'], isset($paramDef['inside']) ? $paramDef['inside'] : null);
                  if (!array_key_exists('inside', $paramDef))
                      $result[$paramDef['name']] = $value;
                  else {
                      if (!isset($result[$paramDef['inside']]))
                          $result[$paramDef['inside']] = array();
                      $result[$paramDef['inside']][$paramDef['name']] = $value;
                  }
              }
          }
      }
      return $result;
  }

  function addSelectToForm(& $form, $settings, $param, $title, $default, $options, $inside, $urlHelp) {
      $form[$param] = array(
        '#type' => 'select',
        '#title' => $title,
        '#options' => $options,
        '#default_value' => $this->getConfigParam($settings, $param, $default, 'str', $inside),
        '#attributes' => array('data-url-help' => $urlHelp == '' ? '' : ($urlHelp . '#' . $param), 'data-param-name' => $param)
      );
  }

  function addTextboxToForm(& $form, $settings, $param, $title, $default, $desc, $inside, $urlHelp) {
        $form[$param] = array(
          '#type' => 'textfield',
          '#title' => $title,
          '#default_value' => $this->getConfigParam($settings, $param, $default, 'str', $inside),
          '#description' => $desc,
          '#attributes' => array('data-url-help' => $urlHelp == '' ? '' : ($urlHelp . '#' . $param), 'data-param-name' => $param)
        );
  }

  function addTextareaToForm(& $form, $settings, $param, $title, $default, $desc, $inside, $urlHelp) {
        $form[$param] = array(
          '#type' => 'textarea',
          '#title' => $title,
          '#default_value' => $this->getConfigParam($settings, $param, $default, 'str', $inside),
          '#description' => $desc,
          '#attributes' => array('data-url-help' => $urlHelp == '' ? '' : ($urlHelp . '#' . $param), 'data-param-name' => $param)
        );
  }

  function addCheckboxToForm(& $form, $settings, $param, $title, $default, $desc, $inside, $urlHelp) {
        $form[$param] = array(
          '#type' => 'checkbox',
          '#title' => $title,
          '#default_value' => $this->getConfigParam($settings, $param, $default, 'bool', $inside),
          '#description' => $desc,
          '#attributes' => array('data-url-help' => $urlHelp == '' ? '' : ($urlHelp . '#' . $param), 'data-param-name' => $param)
        );
  }

  public function isInstalled() {
    return file_exists($_SERVER['DOCUMENT_ROOT'] . '/libraries/jsplus_backup');
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state, Editor $editor) {
    $editor_settings = $editor->getSettings();
    if (isset($editor_settings['plugins']['jsplus_backup']))
      $settings = $editor_settings['plugins']['jsplus_backup'];

    $form['#attached']['library'][] = 'ckeditor_backup/ckeditor_backup.admin';

    if (!$this->isInstalled()) {
        $form['warning'] = array(
            '#markup' => 'Looks like CKEditor Backup (Drupal 8 module) is installed but CKEditor add-ons not found. In order to use this module please copy CKEditor add-ons into "libraries" forder in the root of your Drupal 8 installation (create the directory if it does not exist).'
        );
        return $form;
    }

    $params = array();
    foreach ($this->plugins as $pluginName => $pluginDef) {
        if (isset($pluginDef['params']))
            foreach ($pluginDef['params'] as $paramDef) {
                $paramDef['urlDoc'] = $pluginDef['urlDoc'];
                $params[$paramDef['order']] = $paramDef;
            }
    }
    ksort($params); // sort by key (order)

    if (count($params) > 0) {
        foreach ($params as $order => $paramDef) {
            $inside = isset($paramDef['inside']) ? $paramDef['inside'] : null;
            if ($paramDef['type'] == 'json')
                $paramDef['title'] = $paramDef['title'] . "<br/>\nImportant notice: this value needs to be in JSON format. So this is the must to use double quotes (not single ones) for all key names and all string values";
            if ($paramDef['widget'] == 'select') {
                $this->addSelectToForm($form, $settings, $paramDef['name'], t($paramDef['title']), $paramDef['default'], $paramDef['widgetOptions'], $inside, $paramDef['urlDoc']);
            } else if ($paramDef['widget'] == 'checkbox') {
                $this->addCheckboxToForm($form, $settings, $paramDef['name'], t($paramDef['title']), $paramDef['default'], $paramDef['hint'], $inside, $paramDef['urlDoc']);
            } else if ($paramDef['widget'] == 'textarea') {
                $this->addTextareaToForm($form, $settings, $paramDef['name'], t($paramDef['title']), $paramDef['default'], $paramDef['hint'], $inside, $paramDef['urlDoc']);
            } else {
                $this->addTextboxToForm($form, $settings, $paramDef['name'], t($paramDef['title']), $paramDef['default'], $paramDef['hint'], $inside, $paramDef['urlDoc']);
            }
        }
    } else {
        //$settings = array();
        $form['warning'] = array(
            '#markup' => 'This add-on does not have parameters.'
        );
    }

    return $form;
  }

}
